C  Program to generate distribution based on kriging of control points
C  Willem A. Schreuder
C  October 10, 2000
C  This program was developed with funds provided by the 
C  Rio Grande Water Conservation District
C  for use in the Rio Grande Decision Support System
C  Adapted for Republican River
      PROGRAM MKLK
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NROW=165,NCOL=326,MAXDR=3,MAXP=128,MAXF=128,MAXFP=1024)
      PARAMETER (X0=266023,Y0=14092806,DX=5280,DY=5280)
      DIMENSION X(NCOL),Y(NROW)
      DIMENSION XP(MAXP),YP(MAXP),ZP(MAXP)
      DIMENSION XV(MAXP),YV(MAXP),ZV(MAXP)
      DIMENSION XF(MAXFP),YF(MAXFP),KF(0:MAXF)
      DIMENSION IBOUND(NCOL,NROW)
      DIMENSION FIX(NCOL,NROW),Z(NCOL,NROW),Q(NCOL,NROW)
      LOGICAL       VISIBLE
      CHARACTER*10  CMD
      CHARACTER*256 CHARIN,FILE,LINE,OUTFILE

C  Set defaults
      OUTFILE = 'xxx.out'
      N   = 0
      NF  = 0
      NFP = 0
      LK  = 0
      IDR = 0
      KF(0) = 0
      do I=1,NROW
        do J=1,NCOL
          IBOUND(J,I) = 1
          FIX(J,I)    = 0
          Q(J,I)      = 1
        enddo
      enddo
C  Read parameters
      if (IARGC().ne.1) call ERROR('Usage: mklk <parameter file>')
      call GETARG(1,FILE)
      OPEN(1,FILE=FILE,STATUS='OLD',IOSTAT=IOS)
      if (IOS.ne.0) call ERROR('Error opening parameter file')
100   READ(1,'(A)',END=999) LINE
      if (LINEFIX(LINE).ne.0) goto 100
      CMD = CHARIN(LINE,1,IPOS,IOS)
      if (IOS.ne.0) call ERROR('Error reading command '//LINE)
C  Set output file name
      if (CMD.eq.'OUTPUT') then
        OUTFILE = CHARIN(LINE,IPOS,IPOS,IOS)
        if (IOS.ne.0) call ERROR('Error reading OUTPUT name '//LINE)
C  Read IBOUNDs
      elseif (CMD.eq.'IBOUND') then
        FILE = CHARIN(LINE,IPOS,IPOS,IOS)
        if (IOS.ne.0) call ERROR('Error reading IBOUND name '//LINE)
        call READFLAGS(FILE,IBOUND,NROW,NCOL,LINE,N,0)
C  Read FIX file
      elseif (CMD.eq.'FIX') then
        FILE = CHARIN(LINE,IPOS,IPOS,IOS)
        if (IOS.ne.0) call ERROR('Error reading FIX name '//LINE)
        call READREAL(FILE,FIX,NROW,NCOL,0)
C  Read multiplier file
      elseif (CMD.eq.'MUL') then
        FILE = CHARIN(LINE,IPOS,IPOS,IOS)
        if (IOS.ne.0) call ERROR('Error reading MULtiplier name '//LINE)
        call READREAL(FILE,Q,NROW,NCOL,0)
C  Logarithmic kriging
      elseif (CMD.eq.'LOG') then
        LK = 1
C  Drift
      elseif (CMD.eq.'DRIFT') then
        IDR = INTIN(LINE,IPOS,IPOS,IOS)
        if (IOS.ne.0 .or. IDR.lt.0 .or. IDR.gt.MAXDR)
     |    call ERROR('Invalid drift order')
C  Read CONTROL points
      elseif (CMD.eq.'CONTROL') then
200     READ(1,*,ERR=299,END=999) XX,YY,ZZ
        N = N+1
        if (N.gt.MAXP) call ERROR('Too many CONTROL points')
        XP(N) = XX
        YP(N) = YY
        ZP(N) = ZZ
        goto 200
299     BACKSPACE(1)
C  Read FAULT lines
      elseif (CMD.eq.'FAULT') then
        NF = NF+1
        if (NF.gt.MAXF) call ERROR('Too many FAULTs')
300     READ(1,*,ERR=399,END=999) XX,YY
        NFP = NFP+1
        if (NFP.gt.MAXFP) call ERROR('Too many FAULT points')
        XF(NFP) = XX
        YF(NFP) = YY
        KF(NF) = NFP
        goto 300
399     BACKSPACE(1)
C  Duh!
      else
        call ERROR('Unknown parameter '//CMD)
      endif
      goto 100
999   CLOSE(1)

C  Convert to logs
      if (LK.gt.0) then
        do L=1,N
          ZP(L) = LOG(ZP(L))
        enddo
      endif

C  Generate cell coordinates
      do j=1,NCOL
        X(j) = (j-0.5)*DX+X0
      enddo
      do i=1,NROW
        Y(i) = (NROW-i+0.5)*DY+Y0
      enddo

C  Loop over domain
      do I=1,NROW
        do J=1,NCOL
C         Outside domain
          if (IBOUND(J,I).eq.0) then
            Z(J,I) = 0
C         Mannual override
          else if (FIX(J,I).ne.0) then
            Z(J,I) = FIX(J,I)
C         Interpolate
          else
C           List of visible points
            M = 0
            do L=1,N
              if (VISIBLE(X(J),Y(I),XP(L),YP(L),XF,YF,KF,NF)) then
                M = M+1
                XV(M) = XP(L)
                YV(M) = YP(L)
                ZV(M) = ZP(L)
              endif
            enddo
            if (M.eq.0) then
              print *,'Warning:  Data masked at ',I,J
              Z(J,I) = 0
            else
C             Disable drift if too many degrees of freedom
              if (M.le.2*IDR) then
                JDR = 0
              else
                JDR = IDR
              endif
C             Krige to point
              F = POINTKRIGE(X(J),Y(I),X0,Y0,XV,YV,ZV,M,JDR)
              if (LK.gt.0) then
                Z(J,I) = exp(F)
              else
                Z(J,I) = F
              endif
              if (Z(J,I).gt.1000) print *,M,JDR,Z(J,I)
            endif
          endif
        enddo
      enddo
C  Apply multiplier
      do I=1,NROW
        do J=1,NCOL
          Z(J,I) = Z(J,I)*Q(J,I)
        enddo
      enddo
C  Save
      OPEN(8,FILE=OUTFILE,STATUS='UNKNOWN')
      do I=1,NROW
        WRITE(8,'(999F6.1)') (Z(J,I),J=1,NCOL)
      enddo
      CLOSE(8)
      END
C  ------------------------------------------------------------------  C
C  ------------------  Test if line crosses fault  ------------------  C
C  ------------------------------------------------------------------  C
      FUNCTION VISIBLE(X1,Y1,X2,Y2,XF,YF,KF,NF)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      LOGICAL VISIBLE
      DIMENSION XF(1),YF(1),KF(0:NF)

      VISIBLE = .FALSE.
      do i=1,NF
        do j=KF(i-1)+2,KF(i)
          if (INTERSECT(XF(j-1),YF(j-1),XF(j),YF(j),
     |        X1,Y1,X2,Y2,xl,yl,zl).ge.0) RETURN
        enddo
      enddo
      VISIBLE = .TRUE.
      END
C  ------------------------------------------------------------------- C
C  -----------  Determine if two line segments intersect  ------------ C
C  ------------------------------------------------------------------- C
      FUNCTION INTERSECT(X1,Y1,X2,Y2,X3,Y3,X4,Y4,X,Y,F)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (tiny=5.0E-7,zero=0.0+tiny,one=1.0-tiny)
C  INTERSECT return codes:
C     -1 : No intersection
C      0 : Intersection at X,Y
C     +1 : Non-unique intersection

      F = 0
      if (MAX(X1,X2).lt.MIN(X3,X4) .or. MIN(X1,X2).gt.MAX(X3,X4) .or.
     |    MAX(Y1,Y2).lt.MIN(Y3,Y4) .or. MIN(Y1,Y2).gt.MAX(Y3,Y4)) then
        INTERSECT = -1
      ELSE
        x1_x2 = X1-X2
        y1_y2 = Y1-Y2
        x4_x3 = X4-X3
        y4_y3 = Y4-Y3

        det = x1_x2*y4_y3-x4_x3*y1_y2
        gam = (abs(x1_x2)+abs(x4_x3)+abs(y1_y2)+abs(y4_y3))/4
C  Segments on crossing lines
        if (abs(det).gt.tiny*gam) then
          x4_x2 = X4-X2
          y4_y2 = Y4-Y2
          f1 = (y4_y3*x4_x2-x4_x3*y4_y2)/det
          f2 = (x1_x2*y4_y2-y1_y2*x4_x2)/det
          if (zero.lt.f1.and.f1.lt.one.and.zero.lt.f2.and.f2.lt.one)then
            INTERSECT = 0
            F = f1
            X = f1*(x1_x2) + X2
            Y = f1*(y1_y2) + Y2
          else
            INTERSECT = -1
          endif
C  Segments on the same line
        else if (abs(x1_x2*(y3-y1)-y1_y2*(x3-x1)).lt.zero) then
          if (abs(x1_x2).lt.tiny*abs(x1+x2)) then
            if     (y3.lt.y1.and.y1.lt.y4.or.y4.lt.y1.and.y1.lt.y3) then
              INTERSECT = +1
              X = X1
              Y = Y1
            elseif (y3.lt.y2.and.y2.lt.y4.or.y4.lt.y2.and.y2.lt.y3) then
              INTERSECT = +1
              X = X2
              Y = Y2
            elseif (y1.lt.y3.and.y3.lt.y2.or.y2.lt.y3.and.y3.lt.y1) then
              INTERSECT = +1
              X = X3
              Y = Y3
            elseif (y1.lt.y4.and.y4.lt.y2.or.y2.lt.y4.and.y4.lt.y1) then
              INTERSECT = +1
              X = X4
              Y = Y4
            else
              INTERSECT = -1
            endif
          else
            if     (x3.lt.x1.and.x1.lt.x4.or.x4.lt.x1.and.x1.lt.x3) then
              INTERSECT = +1
              X = X1
              Y = Y1
            elseif (x3.lt.x2.and.x2.lt.x4.or.x4.lt.x2.and.x2.lt.x3) then
              INTERSECT = +1
              X = X2
              Y = Y2
            elseif (x1.lt.x3.and.x3.lt.x2.or.x2.lt.x3.and.x3.lt.x1) then
              INTERSECT = +1
              X = X3
              Y = Y3
            elseif (x1.lt.x4.and.x4.lt.x2.or.x2.lt.x4.and.x4.lt.x1) then
              INTERSECT = +1
              X = X4
              Y = Y4
            else
              INTERSECT = -1
            endif
          endif
C  Segments on different parallel lines
        else
          INTERSECT = -1
        endif
      endif
      END
